//
// Cforall Version 1.0.0 Copyright (C) 2015 University of Waterloo
//
// The contents of this file are covered under the licence agreement in the
// file "LICENCE" distributed with Cforall.
//
// SemanticError.h -- 
//
// Author           : Richard C. Bilson
// Created On       : Mon May 18 07:44:20 2015
// Last Modified By : Peter A. Buhr
// Last Modified On : Tue May 19 07:22:23 2015
// Update Count     : 1
//

#ifndef SEMANTICERROR_H
#define SEMANTICERROR_H

#include <exception>
#include <string>
#include <strstream>
#include <list>
#include <iostream>

class SemanticError : public std::exception {
  public:
	SemanticError();
	SemanticError( std::string error );
	template< typename T > SemanticError( const std::string &error, const T *obj );
	~SemanticError() throw() {}

	void append( SemanticError &other );
	bool isEmpty() const;
	void print( std::ostream &os );

	// constructs an exception using the given message and the printed
	// representation of the obj (T must have a print method)
  private:
	std::list< std::string > errors;
};

template< typename T >
SemanticError::SemanticError( const std::string &error, const T *obj ) {
	std::ostrstream os;
	os << "Error: " << error;
	obj->print( os );
	errors.push_back( std::string( os.str(), os.pcount() ) );
}

#endif // SEMANTICERROR_H

// Local Variables: //
// tab-width: 4 //
// mode: c++ //
// compile-command: "make install" //
// End: //
